@@ -15,19 +15,13 @@ class Node:
 
 	#region Constructor
 
-	def __init__(self, parentNode, name, type):
+	def __init__(self, name, type):
 		"""
 		Initializes a new instance of this class.
 		"""
 
 		#region Instance fields
 
-		self.parentNode = parentNode
-		"""A higher region in hierarchy which this region will feed-forward it. This field must be ignored in XML serialization because it causes circular reference error."""
-
-		self.children = []
-		"""All lower regions/sensors in hierarchy which this region will receive feed-forward input."""
-
 		self.name = name
 		"""The name of the Node."""
 
@@ -86,24 +80,4 @@ class Node:
 
 		return self._output
 
-	def addChild(self, node):
-		"""
-		Add a new child to this node.
-		"""
-
-		self.children.append(node)
-
-	def delete(self):
-		"""
-		Delete this node and its child nodes from tree.
-		"""
-
-		# If this region is parent of lower regions/sensors in the hierarchy then delete their nodes.
-		for child in self.children:
-			del child
-
-		# Delete this node from parent's children.
-		self.parentNode.children.remove(self)
-
 	#endregion
-
